@@ -1,4 +1,4 @@
-## $Id: gen-scm.pl,v 1.13 2000/12/16 03:42:44 trow Exp $
+## $Id: gen-scm.pl,v 1.14 2001/11/19 06:06:07 trow Exp $
 
 ##
 ## gen-scm.pl
@@ -295,7 +295,7 @@ sub h_generate {
 
     if (option("includes")) {
 	unshift(@h_inc, @{option("includes")});
-	unshift(@h_inc, qw( <gnome.h> <guppi-guile.h> ));
+	unshift(@h_inc, qw( <gnome.h> "guppi-guile.h" ));
     }
 
     foreach my $x (@h_inc) {
