@@ -1,5 +1,5 @@
 /* This is -*- C -*- */
-/* $Id: guppi-object-barchart.c,v 1.3 2001/11/20 05:16:39 trow Exp $ */
+/* $Id: guppi-object-barchart.c,v 1.4 2001/11/21 03:38:41 trow Exp $ */
 
 /*
  * guppi-object-barchart.c
@@ -515,7 +515,7 @@ update (GuppiObject *obj)
 }
 
 
-static GuppiElementView *
+static gpointer
 build (GuppiObject *obj, double hsize, double vsize)
 {
   GuppiObjectBarchart *obar;
@@ -738,7 +738,7 @@ clicked_box_cb (GuppiCanvasItem *item,
 }
 
 static void
-item_init (GuppiObject *obj, GuppiCanvasItem *item)
+item_init (GuppiObject *obj, GnomeCanvasItem *item)
 {
   GuppiObjectBarchart *obar = GUPPI_OBJECT_BARCHART (obj);
   GuppiCanvasGroup *grp_item = GUPPI_CANVAS_GROUP (item);
@@ -865,4 +865,4 @@ guppi_object_barchart_new (void)
 
 
 
-/* $Id: guppi-object-barchart.c,v 1.3 2001/11/20 05:16:39 trow Exp $ */
+/* $Id: guppi-object-barchart.c,v 1.4 2001/11/21 03:38:41 trow Exp $ */
