@@ -1,5 +1,5 @@
 /* This is -*- C -*- */
-/* $Id: guppi-object-pie.c,v 1.1 2001/10/08 04:17:29 trow Exp $ */
+/* $Id: guppi-object-pie.c,v 1.2 2001/11/21 03:38:41 trow Exp $ */
 
 /*
  * guppi-object-pie.c
@@ -434,7 +434,7 @@ update (GuppiObject *obj)
   guppi_unref (pie_colors);  
 }
 
-static GuppiElementView *
+static gpointer
 build (GuppiObject *obj, double hsize, double vsize)
 {
   GuppiObjectPie *opie;
@@ -589,7 +589,7 @@ clicked_box_cb (GuppiCanvasItem *item, gint box,
 }
 
 static void
-item_init (GuppiObject *obj, GuppiCanvasItem *item)
+item_init (GuppiObject *obj, GnomeCanvasItem *item)
 {
   GuppiObjectPie *pieobj = GUPPI_OBJECT_PIE (obj);
   GuppiCanvasGroup *grp_item = GUPPI_CANVAS_GROUP (item);
@@ -708,4 +708,4 @@ GtkType guppi_object_pie_get_type (void)
   return guppi_object_pie_type;
 }
 
-/* $Id: guppi-object-pie.c,v 1.1 2001/10/08 04:17:29 trow Exp $ */
+/* $Id: guppi-object-pie.c,v 1.2 2001/11/21 03:38:41 trow Exp $ */
