@@ -26,6 +26,8 @@
  */
 
 #include <config.h>
+#include "guppi-date-series.h"
+
 #include <stdlib.h>
 
 #include <libgnome/gnome-defs.h>
@@ -33,34 +35,8 @@
 #include <libgnome/gnome-i18n.h>
 
 #include <guppi-convenient.h>
-#include "guppi-date-series.h"
-#include "guppi-date-series-impl.h"
 
-static GtkObjectClass * parent_class = NULL;
-
-enum {
-  ARG_0
-};
-
-static void
-guppi_date_series_get_arg (GtkObject *obj, GtkArg *arg, guint arg_id)
-{
-  switch (arg_id) {
-
-  default:
-    break;
-  };
-}
-
-static void
-guppi_date_series_set_arg (GtkObject *obj, GtkArg *arg, guint arg_id)
-{
-  switch (arg_id) {
-
-  default:
-    break;
-  };
-}
+static GtkObjectClass *parent_class = NULL;
 
 static void
 guppi_date_series_finalize (GtkObject *obj)
@@ -77,33 +53,28 @@ export_xml_element (GuppiDateIndexed *dind, const GDate *dt, GuppiXMLDocument *d
   return guppi_xml_new_text_nodef (doc, "scalar", "%g", x);
 }
 
-static void
+static gboolean
 import_xml_element (GuppiDateIndexed *dind, const GDate *dt, GuppiXMLDocument *doc, xmlNodePtr node)
 {
   gchar *s;
   if (strcmp (node->name, "scalar")) {
     g_warning ("improper element type \"%s\"", node->name);
-    return;
+    return FALSE;
   }
   s = xmlNodeListGetString (doc->doc, node->xmlChildrenNode, 1);
   guppi_date_series_set (GUPPI_DATE_SERIES (dind), (GDate *) dt, atof (s));
   free (s);
+  return TRUE;
 }
 
 static void
 guppi_date_series_class_init (GuppiDateSeriesClass *klass)
 {
   GtkObjectClass *object_class = (GtkObjectClass *)klass;
-  GuppiDataClass *data_class = GUPPI_DATA_CLASS (klass);
   GuppiDateIndexedClass *dind_class = GUPPI_DATE_INDEXED_CLASS (klass);
 
   parent_class = gtk_type_class (GUPPI_TYPE_DATE_INDEXED);
 
-  data_class->type_name = _("Date Series");
-  data_class->impl_type = GUPPI_TYPE_DATE_SERIES_IMPL;
-
-  object_class->get_arg = guppi_date_series_get_arg;
-  object_class->set_arg = guppi_date_series_set_arg;
   object_class->finalize = guppi_date_series_finalize;
 
   dind_class->export_xml_element = export_xml_element;
@@ -134,89 +105,74 @@ guppi_date_series_get_type (void)
   return guppi_date_series_type;
 }
 
-GuppiData *
-guppi_date_series_new (void)
-{
-  return guppi_data_newv (GUPPI_TYPE_DATE_SERIES, NULL, 0, NULL); 
-}
-
 double
-guppi_date_series_get (GuppiDateSeries *ser, GDate *dt)
+guppi_date_series_get (GuppiDateSeries *ser, const GDate *dt)
 {
-  GuppiDateSeriesImpl *impl;
-  GuppiDateSeriesImplClass *impl_class;
+  GuppiDateSeriesClass *klass;
 
   g_return_val_if_fail (GUPPI_IS_DATE_SERIES (ser), 0);
-  g_return_val_if_fail (dt && g_date_valid (dt), 0);
+  g_return_val_if_fail (dt && g_date_valid ((GDate *)dt), 0);
 
-  impl = GUPPI_DATE_SERIES_IMPL (guppi_data_impl (GUPPI_DATA (ser)));
-  impl_class = GUPPI_DATE_SERIES_IMPL_CLASS (GTK_OBJECT (impl)->klass);
+  klass = GUPPI_DATE_SERIES_CLASS (GTK_OBJECT (ser)->klass);
   
-  g_assert (impl_class->get);
+  g_assert (klass->get);
 
-  return impl_class->get (impl, dt);
+  return klass->get (ser, dt);
 }
 
 void
-guppi_date_series_set (GuppiDateSeries *ser, GDate *dt, double x)
+guppi_date_series_set (GuppiDateSeries *ser, const GDate *dt, double x)
 {
-  GuppiDateSeriesImpl *impl;
-  GuppiDateSeriesImplClass *impl_class;
+  GuppiDateSeriesClass *klass;
 
   g_return_if_fail (GUPPI_IS_DATE_SERIES (ser));
   g_return_if_fail (guppi_data_can_change (GUPPI_DATA (ser)));
-  g_return_if_fail (dt && g_date_valid (dt));
+  g_return_if_fail (dt && g_date_valid ((GDate *) dt));
 
-  impl = GUPPI_DATE_SERIES_IMPL (guppi_data_impl (GUPPI_DATA (ser)));
-  impl_class = GUPPI_DATE_SERIES_IMPL_CLASS (GTK_OBJECT (impl)->klass);
+  klass = GUPPI_DATE_SERIES_CLASS (GTK_OBJECT (ser)->klass);
 
-  g_assert (impl_class->set);
+  g_assert (klass->set);
 
-  impl_class->set (impl, dt, x);
+  klass->set (ser, dt, x);
 }
 
 void
-guppi_date_series_unset (GuppiDateSeries *ser, GDate *dt)
+guppi_date_series_unset (GuppiDateSeries *ser, const GDate *dt)
 {
-  GuppiDateSeriesImpl *impl;
-  GuppiDateSeriesImplClass *impl_class;
+  GuppiDateSeriesClass *klass;
 
   g_return_if_fail (GUPPI_IS_DATE_SERIES (ser));
   g_return_if_fail (guppi_data_can_change (GUPPI_DATA (ser)));
-  g_return_if_fail (dt && g_date_valid (dt));
+  g_return_if_fail (dt && g_date_valid ((GDate *) dt));
 
-  impl = GUPPI_DATE_SERIES_IMPL (guppi_data_impl (GUPPI_DATA (ser)));
-  impl_class = GUPPI_DATE_SERIES_IMPL_CLASS (GTK_OBJECT (impl)->klass);
+  klass = GUPPI_DATE_SERIES_CLASS (GTK_OBJECT (ser)->klass);
 
-  g_assert (impl_class->unset);
+  g_assert (klass->unset);
 
-  impl_class->unset (impl, dt);
+  klass->unset (ser, dt);
 }
 
 gint
 guppi_date_series_get_many (GuppiDateSeries *ser,
-			    GDate *base_date, gint count,
+			    const GDate *base_date, gint count,
 			    double *buffer)
 {
-  GuppiDateSeriesImpl *impl;
-  GuppiDateSeriesImplClass *impl_class;
+  GuppiDateSeriesClass *klass;
   gint retval;
   GDate dt;
   gint j;
 
-  g_return_val_if_fail (ser && GUPPI_IS_DATE_SERIES (ser), 0);
-  g_return_val_if_fail (base_date && g_date_valid (base_date), 0);
+  g_return_val_if_fail (GUPPI_IS_DATE_SERIES (ser), 0);
+  g_return_val_if_fail (base_date && g_date_valid ((GDate *) base_date), 0);
 
   if (count == 0)
     return 0;
 
   g_return_val_if_fail (buffer, 0);
 
-  impl = GUPPI_DATE_SERIES_IMPL (guppi_data_impl (GUPPI_DATA (ser)));
-  impl_class = GUPPI_DATE_SERIES_IMPL_CLASS (GTK_OBJECT (impl)->klass);
+  klass = GUPPI_DATE_SERIES_CLASS (GTK_OBJECT (ser)->klass);
 
-  if (impl_class->get_many
-      && impl_class->get_many (impl, base_date, count, buffer, &retval))
+  if (klass->get_many && klass->get_many (ser, base_date, count, buffer, &retval))
     return retval;
 
   dt = *base_date;
@@ -246,12 +202,12 @@ guppi_date_series_get_many (GuppiDateSeries *ser,
 
 gint
 guppi_date_series_get_range (GuppiDateSeries *ser,
-			     GDate *start_date, GDate *end_date,
+			     const GDate *start_date, const GDate *end_date,
 			     double *buffer, gint bufsize)
 {
-  g_return_val_if_fail (ser && GUPPI_IS_DATE_SERIES (ser), 0);
-  g_return_val_if_fail (start_date && g_date_valid (start_date), 0);
-  g_return_val_if_fail (end_date && g_date_valid (end_date), 0);
+  g_return_val_if_fail (GUPPI_IS_DATE_SERIES (ser), 0);
+  g_return_val_if_fail (start_date && g_date_valid ((GDate *) start_date), 0);
+  g_return_val_if_fail (end_date && g_date_valid ((GDate *) end_date), 0);
 
   if (bufsize == 0)
     return 0;
@@ -264,46 +220,50 @@ guppi_date_series_get_range (GuppiDateSeries *ser,
 
 gint
 guppi_date_series_get_range_timecoded (GuppiDateSeries *ser,
-				       GDate *start_date, GDate *end_date,
+				       const GDate *start_date, const GDate *end_date,
 				       double *timecode, double *buffer,
 				       gint bufsize)
 {
-  GuppiDateSeriesImpl *impl;
-  GuppiDateSeriesImplClass *impl_class;
+  GuppiDateSeriesClass *klass;
   gint retval;
-  GDate dt;
+  GDate sd, ed, dt;
   gint j;
 
-  g_return_val_if_fail (ser && GUPPI_IS_DATE_SERIES (ser), 0);
-  g_return_val_if_fail (start_date && g_date_valid (start_date), 0);
-  g_return_val_if_fail (end_date && g_date_valid (end_date), 0);
+  g_return_val_if_fail (GUPPI_IS_DATE_SERIES (ser), 0);
+  g_return_val_if_fail (start_date && g_date_valid ((GDate *) start_date), 0);
+  g_return_val_if_fail (end_date && g_date_valid ((GDate *) end_date), 0);
 
   if (bufsize == 0)
     return 0;
 
   g_return_val_if_fail (buffer || timecode, 0);
 
-  guppi_date_indexed_clamp (GUPPI_DATE_INDEXED (ser), start_date);
-  guppi_date_indexed_clamp (GUPPI_DATE_INDEXED (ser), end_date);
-  guppi_2sort_dt (&start_date, &end_date);
+  sd = *start_date;
+  ed = *end_date;
 
-  impl = GUPPI_DATE_SERIES_IMPL (guppi_data_impl (GUPPI_DATA (ser)));
-  impl_class = GUPPI_DATE_SERIES_IMPL_CLASS (GTK_OBJECT (impl)->klass);
+  guppi_date_indexed_clamp (GUPPI_DATE_INDEXED (ser), &sd);
+  guppi_date_indexed_clamp (GUPPI_DATE_INDEXED (ser), &ed);
+  if (g_date_gt (&sd, &ed)) {
+    dt = sd;
+    sd = ed;
+    ed = sd;
+  }
+
+  klass = GUPPI_DATE_SERIES_CLASS (GTK_OBJECT (ser)->klass);
 
   /* Try to use an implementation-supplied method. */
-  if (impl_class->get_range
-      && impl_class->get_range (impl, start_date, end_date,
-				timecode, buffer, bufsize, &retval))
+  if (klass->get_range
+      && klass->get_range (ser, &sd, &ed, timecode, buffer, bufsize, &retval))
     return retval;
 
 
-  dt = *start_date;
+  dt = sd;
   j = 0;
 
   if (!guppi_date_indexed_valid (GUPPI_DATE_INDEXED (ser), &dt))
     guppi_date_indexed_incr (GUPPI_DATE_INDEXED (ser), &dt);
 
-  while (g_date_lteq (&dt, end_date) && j < bufsize) {
+  while (g_date_lteq (&dt, &ed) && j < bufsize) {
     
     if (timecode)
       timecode[j] = g_date_julian (&dt);
@@ -320,36 +280,41 @@ guppi_date_series_get_range_timecoded (GuppiDateSeries *ser,
 
 gboolean
 guppi_date_series_get_bounds (GuppiDateSeries *ser,
-			      GDate *start_date, GDate *end_date,
+			      const GDate *start_date, const GDate *end_date,
 			      double *min, double *max)
 {
-  GuppiDateSeriesImpl *impl;
-  GuppiDateSeriesImplClass *impl_class;
+  GuppiDateSeriesClass *klass;
   gboolean retval;
+  GDate sd, ed;
   double *buffer;
   gint i, N;
   double m, M;
 
 
-  g_return_val_if_fail (ser && GUPPI_IS_DATE_SERIES (ser), 0);
-  g_return_val_if_fail (start_date && g_date_valid (start_date), 0);
-  g_return_val_if_fail (end_date && g_date_valid (end_date), 0);
+  g_return_val_if_fail (GUPPI_IS_DATE_SERIES (ser), 0);
+  g_return_val_if_fail (start_date && g_date_valid ((GDate *) start_date), 0);
+  g_return_val_if_fail (end_date && g_date_valid ((GDate *) end_date), 0);
 
-  impl = GUPPI_DATE_SERIES_IMPL (guppi_data_impl (GUPPI_DATA (ser)));
-  impl_class = GUPPI_DATE_SERIES_IMPL_CLASS (GTK_OBJECT (impl)->klass);
+  klass = GUPPI_DATE_SERIES_CLASS (GTK_OBJECT (ser)->klass);
 
-  guppi_date_indexed_clamp (GUPPI_DATE_INDEXED (ser), start_date);
-  guppi_date_indexed_clamp (GUPPI_DATE_INDEXED (ser), end_date);
-  guppi_2sort_dt (&start_date, &end_date);
+  sd = *start_date;
+  ed = *end_date;
+
+  guppi_date_indexed_clamp (GUPPI_DATE_INDEXED (ser), &sd);
+  guppi_date_indexed_clamp (GUPPI_DATE_INDEXED (ser), &ed);
+  if (g_date_gt (&sd, &ed)) {
+    GDate dt = sd;
+    sd = ed;
+    ed = dt;
+  }
 
   /* Try to use an implementation-supplied method. */
-  if (impl_class->get_bounds
-      && impl_class->get_bounds (impl, start_date, end_date, min, max,
-				 &retval))
+  if (klass->get_bounds
+      && klass->get_bounds (ser, &sd, &ed, min, max, &retval))
     return retval;
     
 
-  N = (gint)g_date_julian (end_date) - (gint)g_date_julian (start_date) + 1;
+  N = (gint)g_date_julian (&ed) - (gint)g_date_julian (&ed) + 1;
   buffer = guppi_new (double, N);
   N = guppi_date_series_get_range (ser, start_date, end_date,
 				   buffer, N);
