@@ -1,5 +1,5 @@
 /* This is -*- C -*- */
-/* $Id: guppi-element-view.c,v 1.58 2001/11/20 03:40:57 trow Exp $ */
+/* $Id: guppi-element-view.c,v 1.59 2001/11/21 03:38:40 trow Exp $ */
 
 /*
  * guppi-element-view.c
@@ -1222,8 +1222,20 @@ changed_size (GuppiElementView *view, double width, double height)
 void
 guppi_element_view_changed_size (GuppiElementView *view, double width, double height)
 {
+  GuppiGeometry *geom;
   g_return_if_fail (GUPPI_IS_ELEMENT_VIEW (view));
-  gtk_signal_emit (GTK_OBJECT (view), view_signals[CHANGED_SIZE], width, height);
+
+  if (width < 0)
+    width = -1;
+  if (height < 0)
+    height = -1;
+
+  geom = guppi_element_view_geometry (view);
+
+  if (fabs (guppi_geometry_get_natural_width (geom) - width) > 1e-4
+      || fabs (guppi_geometry_get_natural_height (geom) - height) > 1e-4) {
+    gtk_signal_emit (GTK_OBJECT (view), view_signals[CHANGED_SIZE], width, height);
+  }
 }
 
 void
@@ -1671,4 +1683,4 @@ GtkType guppi_element_view_get_type (void)
 }
 
 
-/* $Id: guppi-element-view.c,v 1.58 2001/11/20 03:40:57 trow Exp $ */
+/* $Id: guppi-element-view.c,v 1.59 2001/11/21 03:38:40 trow Exp $ */
