@@ -1,9 +1,10 @@
-/* $Id: guppi-marker.c,v 1.9 2001/05/06 08:26:43 trow Exp $ */
+/* $Id: guppi-marker.c,v 1.14 2001/11/08 06:13:42 trow Exp $ */
 
 /*
  * guppi-marker.c
  *
  * Copyright (C) 2000 EMC Capital Management, Inc.
+ * Copyright (C) 2001 The Free Software Foundation
  *
  * Developed by Jon Trowbridge <trow@gnu.org>.
  *
@@ -24,6 +25,7 @@
  */
 
 #include <config.h>
+#include "guppi-marker.h"
 
 #include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-config.h>
@@ -32,7 +34,24 @@
 #include <math.h>
 #include <ctype.h>
 #include <guppi-metrics.h>
-#include "guppi-marker.h"
+#include <guppi-memory.h>
+#include "guppi-pixbuf-stock.h"
+
+#define CIRCLE_AREA_FACTOR   1.0
+#define SQUARE_AREA_FACTOR   1.2533141373155002512  /* sqrt(pi / 2) */
+#define TRIANGLE_AREA_FACTOR 1.55512030155621416074 /* sqrt(4 pi / (3 sqrt 3)) */
+
+#define CIRCLE_DEFAULT_RADIUS   (4.0 * CIRCLE_AREA_FACTOR)
+#define SQUARE_DEFAULT_RADIUS   (CIRCLE_DEFAULT_RADIUS * SQUARE_AREA_FACTOR)
+#define TRIANGLE_DEFAULT_RADIUS (CIRCLE_DEFAULT_RADIUS * TRIANGLE_AREA_FACTOR)
+#define STARBURST_DEFAULT_RADIUS 7.0
+
+#define CIRCLE_DEFAULT_WIDTH   (0.20 * CIRCLE_DEFAULT_RADIUS)
+#define SQUARE_DEFAULT_WIDTH   CIRCLE_DEFAULT_WIDTH
+#define TRIANGLE_DEFAULT_WIDTH CIRCLE_DEFAULT_WIDTH
+#define STARBURST_DEFAULT_WIDTH 2.5
+
+#define STARBURST_EDGE_FACTOR 0.45
 
 typedef struct _GuppiMarkerInfoPrivate GuppiMarkerInfoPrivate;
 
@@ -40,6 +59,8 @@ struct _GuppiMarkerInfoPrivate {
   GuppiMarkerInfo info;
 
   GuppiAlphaTemplate *(*make_alpha_template) (double sz1, double sz2);
+  GuppiPixbuf        *(*make_pixbuf)         (double sz1, double sz2);
+
   void (*print) (GuppiElementPrint *, double x, double y,
 		 double sz1, double sz2);
 };
@@ -48,58 +69,85 @@ struct _GuppiMarkerInfoPrivate {
 
 /*** Marker Alpha Template Constructors */
 
-static GuppiAlphaTemplate *
-template_circle (double r, double w)
+static GuppiPixbuf *
+pixbuf_circle (double r, double w)
 {
   r = guppi_pt2px (r);
   w = guppi_pt2px (w);
 
-  return guppi_alpha_template_new_ring (r, w);
+  return guppi_pixbuf_stock_new_circle (r, w);
+}
+
+static GuppiPixbuf *
+pixbuf_diamond (double r, double w)
+{
+  r = guppi_pt2px (r);
+  w = guppi_pt2px (w);
+
+  return guppi_pixbuf_stock_new_square (r, w, M_PI / 4);
+}
+
+static GuppiPixbuf *
+pixbuf_cross (double r, double w)
+{
+  r = guppi_pt2px (r);
+  w = guppi_pt2px (w);
+
+  return guppi_pixbuf_stock_new_starburst (r, w, STARBURST_EDGE_FACTOR * w, 4, 0);
+}
+
+static GuppiPixbuf *
+pixbuf_x (double r, double w)
+{
+  r = guppi_pt2px (r);
+  w = guppi_pt2px (w);
+
+  return guppi_pixbuf_stock_new_starburst (r, w, STARBURST_EDGE_FACTOR * w, 4, M_PI / 4);
+}
+
+static GuppiPixbuf *
+pixbuf_square (double r, double w)
+{
+  r = guppi_pt2px (r);
+  w = guppi_pt2px (w);
+
+  return guppi_pixbuf_stock_new_square (r, w, 0);
+}
+
+static GuppiPixbuf *
+pixbuf_ast (double r, double w)
+{
+  r = guppi_pt2px (r);
+  w = guppi_pt2px (w);
+
+  return guppi_pixbuf_stock_new_starburst (r, w, STARBURST_EDGE_FACTOR * w, 5, -M_PI/2);
+}
+
+static GuppiPixbuf *
+pixbuf_triangle (double r, double w)
+{
+  r = guppi_pt2px (r);
+  w = guppi_pt2px (w);
+
+  return guppi_pixbuf_stock_new_triangle (r, w, 0);
 }
 
 static GuppiAlphaTemplate *
-template_diamond (double r, double w)
+template_bar (double r, double w)
 {
   r = guppi_pt2px (r);
   w = guppi_pt2px (w);
 
-  return guppi_alpha_template_new_rectangle (r, w, M_PI / 4);
+  return guppi_alpha_template_new_bar (r, r, w, 0);
 }
 
 static GuppiAlphaTemplate *
-template_cross (double r, double w)
+template_half_bar (double r, double w)
 {
   r = guppi_pt2px (r);
   w = guppi_pt2px (w);
 
-  return guppi_alpha_template_new_cross (r, w, 0);
-}
-
-static GuppiAlphaTemplate *
-template_x (double r, double w)
-{
-  r = guppi_pt2px (r);
-  w = guppi_pt2px (w);
-
-  return guppi_alpha_template_new_cross (r, w, M_PI / 4);
-}
-
-static GuppiAlphaTemplate *
-template_square (double r, double w)
-{
-  r = guppi_pt2px (r);
-  w = guppi_pt2px (w);
-
-  return guppi_alpha_template_new_rectangle (r, w, 0);
-}
-
-static GuppiAlphaTemplate *
-template_ast (double r, double w)
-{
-  r = guppi_pt2px (r);
-  w = guppi_pt2px (w);
-
-  return guppi_alpha_template_new_ast (r, w, 0);
+  return guppi_alpha_template_new_bar (r, 0, w, 0);
 }
 
 #if 0
@@ -118,25 +166,32 @@ template_dot (double sz1, double sz2)
 }
 #endif
 
-static GuppiAlphaTemplate *
-template_filled_circle (double r, double unused)
+static GuppiPixbuf *
+pixbuf_filled_circle (double r, double unused)
 {
   r = guppi_pt2px (r);
-  return guppi_alpha_template_new_circle (r);
+  return guppi_pixbuf_stock_new_circle (r, 0);
 }
 
-static GuppiAlphaTemplate *
-template_filled_square (double r, double unused)
+static GuppiPixbuf *
+pixbuf_filled_square (double r, double unused)
 {
   r = guppi_pt2px (r);
-  return guppi_alpha_template_new_box (r, 0);
+  return guppi_pixbuf_stock_new_square (r, 0, 0);
 }
 
-static GuppiAlphaTemplate *
-template_filled_diamond (double r, double unused)
+static GuppiPixbuf *
+pixbuf_filled_diamond (double r, double unused)
 {
   r = guppi_pt2px (r);
-  return guppi_alpha_template_new_box (r, M_PI / 4);
+  return guppi_pixbuf_stock_new_square (r, 0, M_PI / 4);
+}
+
+static GuppiPixbuf *
+pixbuf_filled_triangle (double r, double unused)
+{
+  r = guppi_pt2px (r);
+  return guppi_pixbuf_stock_new_triangle (r, 0, 0);
 }
 
 /**************************************************************************/
@@ -278,6 +333,59 @@ generic_print_square (GuppiElementPrint * ep,
   }
 }
 
+static void
+generic_print_triangle (GuppiElementPrint * ep,
+			double x, double y,
+			double sz, double w,
+			gboolean filled,
+			guint32 fill_color, guint32 edge_color)
+{
+  gint j;
+  g_return_if_fail (ep != NULL);
+  g_return_if_fail (GUPPI_IS_ELEMENT_PRINT (ep));
+
+  for (j = 0; j < 2; ++j) {
+
+    if ((j == 0 && filled) || (j == 1 && w > 0)) {
+
+      guppi_element_print_newpath (ep);
+      guppi_element_print_moveto (ep, x - sz, y - sz);
+      guppi_element_print_lineto (ep, x, y + sz);
+      guppi_element_print_lineto (ep, x + sz, y - sz);
+      guppi_element_print_closepath (ep);
+
+      if (j == 0 && filled) {
+
+	if (fill_color != 0)
+	  guppi_element_print_setrgbacolor_uint (ep, fill_color);
+	guppi_element_print_fill (ep);
+
+      } else if (j == 1 && w > 0) {
+
+	guppi_element_print_setlinewidth (ep, w);
+	guppi_element_print_stroke (ep);
+
+      }
+    }
+  }
+}
+
+static void
+generic_print_bar (GuppiElementPrint * ep,
+		   double x, double y,
+		   double sz, gboolean half,
+		   guint32 edge_color)
+{
+  g_return_if_fail (ep != NULL);
+  g_return_if_fail (GUPPI_IS_ELEMENT_PRINT (ep));
+
+  guppi_element_print_newpath (ep);
+  guppi_element_print_moveto (ep, half ? x : x - sz, y);
+  guppi_element_print_lineto (ep, x + sz, y);
+  guppi_element_print_closepath (ep);
+  guppi_element_print_stroke (ep);
+}
+
 static void
 print_circle (GuppiElementPrint * ep, double x, double y, double r, double w)
 {
@@ -354,6 +462,27 @@ print_ast (GuppiElementPrint * ep, double x, double y, double sz, double w)
   guppi_element_print_stroke (ep);
 }
 
+static void
+print_triangle (GuppiElementPrint * ep, double x, double y,
+		double sz, double unused)
+{
+  generic_print_triangle (ep, x, y, sz, 0, FALSE, 0, 0);
+}
+
+static void
+print_bar (GuppiElementPrint * ep, double x, double y,
+	   double sz, double unused)
+{
+  generic_print_bar (ep, x, y, sz, FALSE, 0);
+}
+
+static void
+print_half_bar (GuppiElementPrint * ep, double x, double y,
+		double sz, double unused)
+{
+  generic_print_bar (ep, x, y, sz, TRUE, 0);
+}
+
 #if 0
 static void
 print_pixel (GuppiElementPrint * ep, double x, double y,
@@ -391,6 +520,13 @@ print_filled_diamond (GuppiElementPrint * ep, double x, double y,
   generic_print_diamond (ep, x, y, sz, 0, TRUE, 0, 0);
 }
 
+static void
+print_filled_triangle (GuppiElementPrint * ep, double x, double y,
+		      double sz, double unused)
+{
+  generic_print_triangle (ep, x, y, sz, 0, TRUE, 0, 0);
+}
+
 /**************************************************************************/
 
 static GuppiMarkerInfoPrivate guppi_marker_info_array[] = {
@@ -398,44 +534,71 @@ static GuppiMarkerInfoPrivate guppi_marker_info_array[] = {
   {{GUPPI_MARKER_NONE, "none", N_("None"),
     NULL, 0, 0, 0,
     NULL, 0, 0, 0},
-   NULL, NULL},
+   NULL, NULL, NULL},
 
   {{GUPPI_MARKER_CIRCLE, "circle", N_("Circle"),
-    N_("Radius"), 0.5, 15, 3,
-    N_("Thickness"), 0.5, 12, 1},
-   template_circle,
+    N_("Radius"), 0.5, 15, CIRCLE_DEFAULT_RADIUS,
+    N_("Thickness"), 0.5, 12, CIRCLE_DEFAULT_WIDTH},
+   NULL,
+   pixbuf_circle,
    print_circle},
 
   {{GUPPI_MARKER_DIAMOND, "diamond", N_("Diamond"),
-    N_("Radius"), 1, 15, 4,
-    N_("Thickness"), 0.5, 12, 1},
-   template_diamond,
+    N_("Radius"), 1, 15, SQUARE_DEFAULT_RADIUS,
+    N_("Thickness"), 0.5, 12, SQUARE_DEFAULT_WIDTH},
+   NULL,
+   pixbuf_diamond,
    print_diamond},
 
   {{GUPPI_MARKER_CROSS, "cross", N_("Cross"),
-    N_("Radius"), 1, 15, 4,
-    N_("Thickness"), 0.5, 12, 1},
-   template_cross,
+    N_("Radius"), 1, 15, STARBURST_DEFAULT_RADIUS,
+    N_("Thickness"), 0.5, 12, STARBURST_DEFAULT_WIDTH},
+   NULL,
+   pixbuf_cross,
    print_cross},
 
   {{GUPPI_MARKER_X, "x", N_("X"),
-    N_("Radius"), 1, 15, 4,
-    N_("Thickness"), 0.5, 12, 1},
-   template_x,
+    N_("Radius"), 1, 15, STARBURST_DEFAULT_RADIUS,
+    N_("Thickness"), 0.5, 12, STARBURST_DEFAULT_WIDTH},
+   NULL,
+   pixbuf_x,
    print_x},
 
   {{GUPPI_MARKER_SQUARE, "square", N_("Square"),
-    N_("Radius"), 1, 15, 4,
-    N_("Thickness"), 0.5, 12, 1},
-   template_square,
+    N_("Radius"), 1, 15, SQUARE_DEFAULT_RADIUS,
+    N_("Thickness"), 0.5, 12, SQUARE_DEFAULT_WIDTH},
+   NULL,
+   pixbuf_square,
    print_square},
 
   {{GUPPI_MARKER_AST, "asterisk", N_("Asterisk"),
-    N_("Radius"), 1, 15, 4,
-    N_("Thickness"), 0.5, 12, 1},
-   template_ast,
+    N_("Radius"), 1, 15, STARBURST_DEFAULT_RADIUS,
+    N_("Thickness"), 0.5, 12, STARBURST_DEFAULT_WIDTH},
+   NULL,
+   pixbuf_ast,
    print_ast},
 
+  {{GUPPI_MARKER_TRIANGLE, "triangle", N_("Triangle"),
+    N_("Radius"), 1, 15, TRIANGLE_DEFAULT_RADIUS,
+    N_("Thickness"), 0.5, 12, TRIANGLE_DEFAULT_WIDTH},
+   NULL,
+   pixbuf_triangle,
+   print_triangle},
+
+  {{GUPPI_MARKER_BAR, "bar", N_("Bar"),
+    N_("Length"), 1, 15, 4,
+    NULL, 0.5, 12, 1},
+   template_bar,
+   NULL,
+   print_bar},
+
+  {{GUPPI_MARKER_HALF_BAR, "half bar", N_("Half Bar"),
+    N_("Length"), 1, 15, 4,
+    NULL, 0.5, 12, 1},
+   template_half_bar,
+   NULL,
+   print_half_bar},
+
 #if 0
   {{GUPPI_MARKER_PIXEL, N_("Pixel"),
     NULL, 0, 0, 0,
@@ -451,22 +614,32 @@ static GuppiMarkerInfoPrivate guppi_marker_info_array[] = {
 #endif
 
   {{GUPPI_MARKER_FILLED_CIRCLE, "filled circle", N_("Filled Circle"),
-    N_("Radius"), 1, 15, 4,
+    N_("Radius"), 1, 15, CIRCLE_DEFAULT_RADIUS,
     NULL, 0, 0, 0},
-   template_filled_circle,
+   NULL,
+   pixbuf_filled_circle,
    print_filled_circle},
 
   {{GUPPI_MARKER_FILLED_SQUARE, "filled square", N_("Filled Square"),
-    N_("Radius"), 1, 15, 4,
+    N_("Radius"), 1, 15, SQUARE_DEFAULT_RADIUS,
     NULL, 0, 0, 0},
-   template_filled_square,
+   NULL,
+   pixbuf_filled_square,
    print_filled_square},
 
   {{GUPPI_MARKER_FILLED_DIAMOND, "filled diamond", N_("Filled Diamond"),
-    N_("Radius"), 1, 15, 4,
+    N_("Radius"), 1, 15, SQUARE_DEFAULT_RADIUS,
     NULL, 0.5, 12, 1},
-   template_filled_diamond,
-   print_filled_diamond}
+   NULL,
+   pixbuf_filled_diamond,
+   print_filled_diamond},
+
+  {{GUPPI_MARKER_FILLED_TRIANGLE, "filled triangle", N_("Filled Triangle"),
+    N_("Radius"), 1, 15, TRIANGLE_DEFAULT_RADIUS,
+    NULL, 0.5, 12, 1},
+   NULL,
+   pixbuf_filled_triangle,
+   print_filled_triangle}
 };
 
 static gboolean
@@ -574,6 +747,43 @@ guppi_marker_alpha_template (GuppiMarker m,
   return NULL;
 }
 
+GuppiPixbuf *
+guppi_marker_pixbuf (GuppiMarker m,
+		     double sz1, double sz2,
+		     double scale_factor)
+{
+  GuppiMarkerInfoPrivate *mipriv;
+
+  g_return_val_if_fail (guppi_marker_valid (m), NULL);
+  g_return_val_if_fail (sz1 >= 0, NULL);
+  g_return_val_if_fail (sz2 >= 0, NULL);
+  g_return_val_if_fail (scale_factor >= 0, NULL);
+
+  if (m == GUPPI_MARKER_NONE)
+    return NULL;
+
+  mipriv = (GuppiMarkerInfoPrivate *) guppi_marker_info (m);
+  g_return_val_if_fail (mipriv != NULL, NULL);
+
+  if (mipriv->make_pixbuf) {
+
+    return mipriv->make_pixbuf (sz1 * scale_factor, sz2 * scale_factor);
+
+  } else if (mipriv->make_alpha_template) {
+
+    GuppiAlphaTemplate *template;
+    GuppiPixbuf *pixbuf;
+    template = mipriv->make_alpha_template (sz1 * scale_factor, sz2 * scale_factor);
+    pixbuf = guppi_pixbuf_new_from_alpha_template (template, 0xff, 0, 0);
+    guppi_unref (template);
+    return pixbuf;
+
+  } 
+
+  g_warning ("No alpha template available for marker %s", mipriv->info.name);
+  return NULL;
+}
+
 void
 guppi_marker_print (GuppiMarker m,
 		    double sz1, double sz2,
@@ -606,4 +816,4 @@ guppi_marker_print (GuppiMarker m,
 
 
 
-/* $Id: guppi-marker.c,v 1.9 2001/05/06 08:26:43 trow Exp $ */
+/* $Id: guppi-marker.c,v 1.14 2001/11/08 06:13:42 trow Exp $ */
