@@ -1,10 +1,11 @@
 /* This is -*- C -*- */
-/* $Id: guppi-view-interval.c,v 1.12 2001/05/06 08:26:43 trow Exp $ */
+/* $Id: guppi-view-interval.c,v 1.18 2001/11/19 05:40:45 trow Exp $ */
 
 /*
  * guppi-view-interval.c
  *
  * Copyright (C) 2000 EMC Capital Management, Inc.
+ * Copyright (C) 2001 The Free Software Foundation
  *
  * Developed by Jon Trowbridge <trow@gnu.org> and
  * Havoc Pennington <hp@pobox.com>.
@@ -26,18 +27,15 @@
  */
 
 #include <config.h>
-#include <math.h>
-
-#include <gtk/gtksignal.h>
-
-#include <guppi-convenient.h>
 #include "guppi-view-interval.h"
 
-static GtkObjectClass *parent_class = NULL;
+#include <stdlib.h>
+#include <math.h>
+#include <gtk/gtksignal.h>
+#include <guppi-convenient.h>
 
-enum {
-  ARG_0
-};
+
+static GtkObjectClass *parent_class = NULL;
 
 enum {
   CHANGED,
@@ -46,33 +44,6 @@ enum {
 };
 static guint gvi_signals[LAST_SIGNAL] = { 0 };
 
-static void
-guppi_view_interval_get_arg (GtkObject * obj, GtkArg * arg, guint arg_id)
-{
-  switch (arg_id) {
-
-  default:
-    break;
-  };
-}
-
-static void
-guppi_view_interval_set_arg (GtkObject * obj, GtkArg * arg, guint arg_id)
-{
-  switch (arg_id) {
-
-  default:
-    break;
-  };
-}
-
-static void
-guppi_view_interval_destroy (GtkObject * obj)
-{
-  if (parent_class->destroy)
-    parent_class->destroy (obj);
-}
-
 static void
 guppi_view_interval_finalize (GtkObject * obj)
 {
@@ -89,9 +60,6 @@ guppi_view_interval_class_init (GuppiViewIntervalClass * klass)
 
   parent_class = gtk_type_class (GTK_TYPE_OBJECT);
 
-  object_class->get_arg = guppi_view_interval_get_arg;
-  object_class->set_arg = guppi_view_interval_set_arg;
-  object_class->destroy = guppi_view_interval_destroy;
   object_class->finalize = guppi_view_interval_finalize;
 
   gvi_signals[CHANGED] =
@@ -115,9 +83,16 @@ guppi_view_interval_class_init (GuppiViewIntervalClass * klass)
 static void
 guppi_view_interval_init (GuppiViewInterval * obj)
 {
+  obj->id = guppi_unique_id ();
   obj->type = GUPPI_VIEW_NORMAL;
   obj->t0 = 0;
   obj->t1 = 1;
+  obj->min = -HUGE_VAL;
+  obj->max = HUGE_VAL;
+  obj->min_width = 0;
+
+  obj->include_min = TRUE;
+  obj->include_max = TRUE;
 }
 
 GtkType guppi_view_interval_get_type (void)
@@ -145,19 +120,25 @@ guppi_view_interval_new (void)
 }
 
 static void
-changed (GuppiViewInterval * v)
+changed (GuppiViewInterval *v)
 {
   if (!v->block_changed_signals)
     gtk_signal_emit (GTK_OBJECT (v), gvi_signals[CHANGED]);
 }
 
 void
-guppi_view_interval_set (GuppiViewInterval * v, double a, double b)
+guppi_view_interval_set (GuppiViewInterval *v, double a, double b)
 {
-  g_return_if_fail (v != NULL);
   g_return_if_fail (GUPPI_IS_VIEW_INTERVAL (v));
 
   guppi_2sort (&a, &b);
+  if (a < v->min)
+    a = v->min;
+  if (b > v->max)
+    b = v->max;
+
+  if (b - a < v->min_width)
+    return;
 
   if (guppi_view_interval_is_logarithmic (v)) {
     if (b <= 0)
@@ -174,9 +155,8 @@ guppi_view_interval_set (GuppiViewInterval * v, double a, double b)
 }
 
 void
-guppi_view_interval_grow_to (GuppiViewInterval * v, double a, double b)
+guppi_view_interval_grow_to (GuppiViewInterval *v, double a, double b)
 {
-  g_return_if_fail (v != NULL);
   g_return_if_fail (GUPPI_IS_VIEW_INTERVAL (v));
 
   if (a > b) {
@@ -193,9 +173,8 @@ guppi_view_interval_grow_to (GuppiViewInterval * v, double a, double b)
 }
 
 void
-guppi_view_interval_range (GuppiViewInterval * v, double *a, double *b)
+guppi_view_interval_range (GuppiViewInterval *v, double *a, double *b)
 {
-  g_return_if_fail (v != NULL);
   g_return_if_fail (GUPPI_IS_VIEW_INTERVAL (v));
 
   if (a)
@@ -204,9 +183,36 @@ guppi_view_interval_range (GuppiViewInterval * v, double *a, double *b)
     *b = v->t1;
 }
 
-gboolean guppi_view_interval_valid_fn (GuppiViewInterval * v, double x)
+void
+guppi_view_interval_set_bounds (GuppiViewInterval *v, double a, double b)
+{
+  g_return_if_fail (GUPPI_IS_VIEW_INTERVAL (v));
+
+  guppi_2sort (&a, &b);
+
+  v->min = a;
+  v->max = b;
+}
+
+void
+guppi_view_interval_clear_bounds (GuppiViewInterval *v)
+{
+  g_return_if_fail (GUPPI_IS_VIEW_INTERVAL (v));
+
+  v->min = -HUGE_VAL;
+  v->max = HUGE_VAL;
+}
+
+void
+guppi_view_interval_set_min_width (GuppiViewInterval *v, double mw)
+{
+  g_return_if_fail (GUPPI_IS_VIEW_INTERVAL (v));
+
+  v->min_width = mw;
+}
+
+gboolean guppi_view_interval_valid_fn (GuppiViewInterval *v, double x)
 {
-  g_return_val_if_fail (v != NULL, FALSE);
   g_return_val_if_fail (GUPPI_IS_VIEW_INTERVAL (v), FALSE);
 
   switch (v->type) {
@@ -226,11 +232,10 @@ gboolean guppi_view_interval_valid_fn (GuppiViewInterval * v, double x)
 }
 
 double
-guppi_view_interval_conv_fn (GuppiViewInterval * v, double x)
+guppi_view_interval_conv_fn (GuppiViewInterval *v, double x)
 {
   double t0, t1;
 
-  g_return_val_if_fail (v != NULL, 0);
   g_return_val_if_fail (GUPPI_IS_VIEW_INTERVAL (v), 0);
 
   t0 = v->t0;
@@ -263,11 +268,10 @@ guppi_view_interval_conv_fn (GuppiViewInterval * v, double x)
 }
 
 double
-guppi_view_interval_unconv_fn (GuppiViewInterval * v, double x)
+guppi_view_interval_unconv_fn (GuppiViewInterval *v, double x)
 {
   double t0, t1;
 
-  g_return_val_if_fail (v != NULL, 0);
   g_return_val_if_fail (GUPPI_IS_VIEW_INTERVAL (v), 0);
 
   t0 = v->t0;
@@ -290,7 +294,7 @@ guppi_view_interval_unconv_fn (GuppiViewInterval * v, double x)
 }
 
 void
-guppi_view_interval_conv_bulk (GuppiViewInterval * v,
+guppi_view_interval_conv_bulk (GuppiViewInterval *v,
 			       const double *in_data,
 			       double *out_data, gsize N)
 {
@@ -298,7 +302,7 @@ guppi_view_interval_conv_bulk (GuppiViewInterval * v,
   gsize i;
   gint type;
 
-  g_return_if_fail (v != NULL && GUPPI_IS_VIEW_INTERVAL (v));
+  g_return_if_fail (GUPPI_IS_VIEW_INTERVAL (v));
   g_return_if_fail (out_data != NULL);
   g_return_if_fail (N == 0 || in_data != NULL);
 
@@ -329,7 +333,7 @@ guppi_view_interval_conv_bulk (GuppiViewInterval * v,
 }
 
 void
-guppi_view_interval_unconv_bulk (GuppiViewInterval * v,
+guppi_view_interval_unconv_bulk (GuppiViewInterval *v,
 				 const double *in_data,
 				 double *out_data, gsize N)
 {
@@ -337,7 +341,7 @@ guppi_view_interval_unconv_bulk (GuppiViewInterval * v,
   gsize i;
   gint type;
 
-  g_return_if_fail (v != NULL && GUPPI_IS_VIEW_INTERVAL (v));
+  g_return_if_fail (GUPPI_IS_VIEW_INTERVAL (v));
   g_return_if_fail (out_data != NULL);
   g_return_if_fail (N == 0 || in_data != NULL);
 
@@ -366,12 +370,11 @@ guppi_view_interval_unconv_bulk (GuppiViewInterval * v,
 }
 
 void
-guppi_view_interval_rescale_around_point (GuppiViewInterval * v,
+guppi_view_interval_rescale_around_point (GuppiViewInterval *v,
 					  double x, double s)
 {
   double a, b;
 
-  g_return_if_fail (v != NULL);
   g_return_if_fail (GUPPI_IS_VIEW_INTERVAL (v));
 
   if (s < 0)
@@ -394,10 +397,9 @@ guppi_view_interval_rescale_around_point (GuppiViewInterval * v,
 }
 
 void
-guppi_view_interval_recenter_around_point (GuppiViewInterval * v, double x)
+guppi_view_interval_recenter_around_point (GuppiViewInterval *v, double x)
 {
   double a, b, c;
-  g_return_if_fail (v != NULL);
   g_return_if_fail (GUPPI_IS_VIEW_INTERVAL (v));
 
   guppi_view_interval_range (v, &a, &b);
@@ -407,25 +409,23 @@ guppi_view_interval_recenter_around_point (GuppiViewInterval * v, double x)
 }
 
 void
-guppi_view_interval_translate (GuppiViewInterval * v, double dx)
+guppi_view_interval_translate (GuppiViewInterval *v, double dx)
 {
   double a, b;
 
-  g_return_if_fail (v != NULL);
   g_return_if_fail (GUPPI_IS_VIEW_INTERVAL (v));
 
-  if (dx != 0) {
+  if (dx != 0  && v->min <= a + dx && b + dx <= v->max) {
     guppi_view_interval_range (v, &a, &b);
     guppi_view_interval_set (v, a + dx, b + dx);
   }
 }
 
 void
-guppi_view_interval_conv_translate (GuppiViewInterval * v, double x)
+guppi_view_interval_conv_translate (GuppiViewInterval *v, double x)
 {
   double a, b;
 
-  g_return_if_fail (v != NULL);
   g_return_if_fail (GUPPI_IS_VIEW_INTERVAL (v));
 
   if (x == 0)
@@ -446,15 +446,17 @@ guppi_view_interval_conv_translate (GuppiViewInterval * v, double x)
 
   b = guppi_view_interval_unconv (v, b);
 
-  guppi_view_interval_set (v, a, b);
+  guppi_2sort (&a, &b);
+
+  if (v->min <= a && b <= v->max)
+    guppi_view_interval_set (v, a, b);
 }
 
 void
-guppi_view_interval_request_preferred_range (GuppiViewInterval * v)
+guppi_view_interval_request_preferred_range (GuppiViewInterval *v)
 {
   double p0, p1;
 
-  g_return_if_fail (v != NULL);
   g_return_if_fail (GUPPI_IS_VIEW_INTERVAL (v));
 
   p0 = v->t0;
@@ -479,9 +481,8 @@ guppi_view_interval_request_preferred_range (GuppiViewInterval * v)
 /**************************************************************************/
 
 void
-guppi_view_interval_scale_linearly (GuppiViewInterval * v)
+guppi_view_interval_scale_linearly (GuppiViewInterval *v)
 {
-  g_return_if_fail (v != NULL);
   g_return_if_fail (GUPPI_IS_VIEW_INTERVAL (v));
 
   if (v->type != GUPPI_VIEW_NORMAL) {
@@ -491,9 +492,8 @@ guppi_view_interval_scale_linearly (GuppiViewInterval * v)
 }
 
 void
-guppi_view_interval_scale_logarithmically (GuppiViewInterval * v, double base)
+guppi_view_interval_scale_logarithmically (GuppiViewInterval *v, double base)
 {
-  g_return_if_fail (v != NULL);
   g_return_if_fail (GUPPI_IS_VIEW_INTERVAL (v));
 
   if (v->type != GUPPI_VIEW_LOG) {
@@ -503,25 +503,106 @@ guppi_view_interval_scale_logarithmically (GuppiViewInterval * v, double base)
   }
 }
 
-gboolean guppi_view_interval_is_logarithmic (const GuppiViewInterval * v)
+gboolean guppi_view_interval_is_logarithmic (const GuppiViewInterval *v)
 {
-  g_return_val_if_fail (v != NULL, FALSE);
   g_return_val_if_fail (GUPPI_IS_VIEW_INTERVAL (v), FALSE);
 
   return v->type == GUPPI_VIEW_LOG;
 }
 
 double
-guppi_view_interval_logarithm_base (const GuppiViewInterval * v)
+guppi_view_interval_logarithm_base (const GuppiViewInterval *v)
 {
-  g_return_val_if_fail (v != NULL, 0);
   g_return_val_if_fail (GUPPI_IS_VIEW_INTERVAL (v), 0);
   g_return_val_if_fail (guppi_view_interval_is_logarithmic (v), 0);
 
   return v->type_arg;
 }
 
+/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */
+
+xmlNodePtr
+guppi_view_interval_export_xml (GuppiViewInterval *vi, GuppiXMLDocument *doc)
+{
+  xmlNodePtr node;
+  gchar buf[64];
+  gchar *s;
+
+  g_return_val_if_fail (GUPPI_IS_VIEW_INTERVAL (vi), NULL);
+  g_return_val_if_fail (doc != NULL, NULL);
+
+  node = xmlNewNode (doc->ns, "ViewInterval");
+
+  s = guppi_uniq2str (vi->id);
+  xmlNewProp (node, "UID", s);
+  guppi_free (s);
+
+  if (! guppi_xml_document_has_cached (doc, vi->id)) {
+    
+    g_snprintf (buf, 64, "%g", vi->t0);
+    xmlNewProp (node, "t0", buf);
+
+    g_snprintf (buf, 64, "%g", vi->t1);
+    xmlNewProp (node, "t1", buf);
+
+    g_snprintf (buf, 64, "%d", vi->type);
+    xmlNewProp (node, "type", buf);
+
+    g_snprintf (buf, 64, "%g", vi->type_arg);
+    xmlNewProp (node, "type_arg", buf);
+
+    guppi_ref (vi);
+    guppi_xml_document_cache_full (doc, vi->id, vi, guppi_unref_fn);
+  }
+
+  return node;
+}
+
+GuppiViewInterval *
+guppi_view_interval_import_xml (GuppiXMLDocument *doc, xmlNodePtr node)
+{
+  GuppiViewInterval *vi;
+  guppi_uniq_t id;
+  gchar *s;
+
+  g_return_val_if_fail (doc != NULL, NULL);
+  g_return_val_if_fail (node != NULL, NULL);
+
+  if (strcmp (node->name, "ViewInterval"))
+    return NULL;
+
+  s = xmlGetProp (node, "UID");
+  id = guppi_str2uniq (s);
+  xmlFree (s);
+
+  vi = guppi_xml_document_lookup (doc, id);
+
+  if (vi && GUPPI_IS_VIEW_INTERVAL (vi))
+    return vi;
+
+  vi = guppi_view_interval_new ();
+  vi->id = id;
+  
+  s = xmlGetProp (node, "t0");
+  vi->t0 = s ? atof (s) : 0;
+  xmlFree (s);
+
+  s = xmlGetProp (node, "t1");
+  vi->t1 = s ? atof (s) : 1;
+  xmlFree (s);
+
+  s = xmlGetProp (node, "type");
+  vi->type = s ? atoi(s) : GUPPI_VIEW_NORMAL;
+  xmlFree (s);
+
+  s = xmlGetProp (node, "type_arg");
+  vi->type_arg = s ? atof (s) : 0;
+  xmlFree (s);
+		
+  guppi_xml_document_cache_full (doc, vi->id, vi, guppi_unref_fn);
+
+  return vi;
+}
 
 
-
-/* $Id: guppi-view-interval.c,v 1.12 2001/05/06 08:26:43 trow Exp $ */
+/* $Id: guppi-view-interval.c,v 1.18 2001/11/19 05:40:45 trow Exp $ */
